Imports System.IO
Imports System.IO.File

Public Class cConnection
    Private mName As String
    Private mFTP As String
    Private mPort As Short
    Private mUserName As String
    Private mPassword As String
    Private mTransfer As Byte
    Private mConnection As Byte
    Private mDefault As Boolean

    Public Enum cnTransfer As Byte
        Binary = 1
        ASCII = 2
    End Enum

    Public Enum cnType As Byte
        Active = 1
        Passive = 2
    End Enum

    Public Property Name() As String
        Get
            Name = mName
        End Get
        Set(ByVal Value As String)
            If Value.Length > 0 Then
                mName = Value
            End If
        End Set
    End Property

    Public Property FTP() As String
        Get
            FTP = mFTP
        End Get
        Set(ByVal Value As String)
            If Value.Length > 0 Then
                mFTP = Value
            End If
        End Set
    End Property

    Public Property Port() As Short
        Get
            Port = mPort
        End Get
        Set(ByVal Value As Short)
            If (Value > 0) And (Value <= 65535) Then
                mPort = Value
            End If
        End Set
    End Property

    Public Property UserName() As String
        Get
            UserName = mUserName
        End Get
        Set(ByVal Value As String)
            mUserName = Value
        End Set
    End Property

    Public Property Password() As String
        Get
            Password = mPassword
        End Get
        Set(ByVal Value As String)
            mPassword = Value
        End Set
    End Property

    Public Property Transfer() As cnTransfer
        Get
            Transfer = mTransfer
        End Get
        Set(ByVal Value As cnTransfer)
            mTransfer = Value
        End Set
    End Property

    Public Property Connection() As cnType
        Get
            Connection = mConnection
        End Get
        Set(ByVal Value As cnType)
            mConnection = Value
        End Set
    End Property

    Public Property DefaultConnection() As Boolean
        Get
            DefaultConnection = mDefault
        End Get
        Set(ByVal Value As Boolean)
            mDefault = Value
        End Set
    End Property

    Public Sub New()
        Port = 21
        Transfer = cnTransfer.Binary
        Connection = cnType.Active
        DefaultConnection = False
    End Sub
End Class

Public Class cCnList
    Implements System.Collections.IEnumerable

    Private mCol As New Collection()

    Public Enum cnTransfer As Byte
        Binary = 1
        ASCII = 2
    End Enum

    Public Enum cnType As Byte
        Active = 1
        Passive = 2
    End Enum

    Public Function Add(ByVal Name As String, ByVal FTP As String, _
     Optional ByVal Port As Short = 21, Optional ByVal UserName As String = "", Optional ByVal Password As String = "", _
     Optional ByVal Transfer As cnTransfer = cnTransfer.Binary, Optional ByVal Type As cnType = cnType.Active, Optional ByVal Def As Boolean = False) As cConnection
        Dim oNewConnection As cConnection

        If Name.Length > 0 And FTP.Length > 0 Then
            oNewConnection = New cConnection()
            With oNewConnection
                .Name = Name
                .FTP = FTP
                .Port = Port
                .UserName = UserName
                .Password = Password
                .Transfer = Transfer
                .Connection = Type
                .DefaultConnection = Def
            End With
            mCol.Add(oNewConnection, Name)
        End If
        Add = oNewConnection
    End Function

    Public Function Add(ByRef Connection As cConnection) As cConnection
        Dim oNewConnection As cConnection

        oNewConnection = New cConnection()
        With oNewConnection
            .Name = Connection.Name
            .FTP = Connection.FTP
            .Port = Connection.Port
            .UserName = Connection.UserName
            .Password = Connection.Password
            .Transfer = Connection.Transfer
            .Connection = Connection.Connection
            .DefaultConnection = Connection.DefaultConnection
        End With
        mCol.Add(oNewConnection, Connection.Name)
        Add = oNewConnection
    End Function

    Public Sub Remove(ByVal Name As String)
        mCol.Remove(Name)
    End Sub

    Public Sub Remove(ByVal Index As Integer)
        mCol.Remove(Index + 1)
    End Sub

    Public Function Clear() As Object
        Dim mIndex As Integer
        If mCol.Count() > 0 Then
            For mIndex = 1 To mCol.Count()
                mCol.Remove(mIndex)
            Next
        End If
    End Function

    Default Public ReadOnly Property Item(ByVal Index As Object) As cConnection
        Get
            If IsNumeric(Index) Then
                Item = mCol.Item(Index + 1)
            Else
                Item = mCol.Item(Index)
            End If
        End Get
    End Property

    Public ReadOnly Property Count() As Integer
        Get
            Count = mCol.Count
        End Get
    End Property

    Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        GetEnumerator = mCol.GetEnumerator
    End Function
End Class

Public Class cCnManager
    Private mCnList As New cCnList()

    Public Event FinishedLoading()

    Public Property Connections() As cCnList
        Get
            Connections = mCnList
        End Get
        Set(ByVal Value As cCnList)

        End Set
    End Property

    Public Function ReadFromFile(ByVal FileName As String) As Integer
        If FileName.Length > 0 Then
            If Dir(FileName).Length > 0 Then
                Dim strLine As String = " "                     ' Must be a non-empty string for the loop below
                Dim ReadStream As StreamReader
                Dim oCn As cConnection
                Dim strTmp As String

                Try
                    ReadStream = File.OpenText(FileName)
                    strLine = ReadStream.ReadLine
                    Do While Not IsNothing(strLine)
                        strLine = strLine.Trim(" ")

                        If strLine.IndexOf("{Begin Connection}") >= 0 Then
                            oCn = New cConnection()
                        ElseIf strLine.IndexOf("Name:") >= 0 Then
                            oCn.Name = strLine.Substring(5)
                            oCn.Name = oCn.Name.Trim(" ")
                        ElseIf strLine.IndexOf("FTP:") >= 0 Then
                            oCn.FTP = strLine.Substring(4)
                            oCn.FTP = oCn.FTP.Trim(" ")
                        ElseIf strLine.IndexOf("Port:") >= 0 Then
                            strTmp = strLine.Substring(5)
                            strTmp = strTmp.Trim(" ")
                            oCn.Port = CShort(strTmp)
                        ElseIf strLine.IndexOf("User:") >= 0 Then
                            oCn.UserName = strLine.Substring(5)
                            oCn.UserName = oCn.UserName.Trim(" ")
                        ElseIf strLine.IndexOf("Password:") >= 0 Then
                            oCn.Password = strLine.Substring(9)
                            oCn.Password = oCn.Password.Trim(" ")
                        ElseIf strLine.IndexOf("Transfer:") >= 0 Then
                            strTmp = strLine.Substring(9)
                            strTmp = strTmp.Trim(" ")
                            oCn.Transfer = IIf(Val(strTmp) = 1, cConnection.cnTransfer.Binary, cConnection.cnTransfer.ASCII)
                        ElseIf strLine.IndexOf("Type:") >= 0 Then
                            strTmp = strLine.Substring(5)
                            strTmp = strTmp.Trim(" ")
                            oCn.Connection = IIf(Val(strTmp) = 1, cConnection.cnType.Active, cConnection.cnType.Passive)
                        ElseIf strLine.IndexOf("Default:") >= 0 Then
                            strTmp = strLine.Substring(8)
                            strTmp = strTmp.Trim(" ")
                            oCn.DefaultConnection = (strTmp.ToUpper = "TRUE")
                        ElseIf strLine.IndexOf("{End Connection}") >= 0 Then
                            If Not IsNothing(oCn) Then
                                mCnList.Add(oCn)
                                oCn = Nothing
                            End If
                        End If
                        strLine = ReadStream.ReadLine
                    Loop
                Catch eof As IO.EndOfStreamException
                    ' no action required
                Catch err As Exception
                    MsgBox("Error while reading the connections file! " & err.Message, MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Exclamation + MsgBoxStyle.OKOnly, "TransData v1.00 (alpha)")
                Finally
                    ReadStream.Close()
                End Try
                RaiseEvent FinishedLoading()
            End If
        End If
    End Function

    Public Function WriteToFile(ByVal FileName As String) As Integer
        Dim WriteStream As StreamWriter
        Dim oCn As cConnection

        If mCnList.Count > 0 Then
            Try
                If Dir(FileName).Length > 0 Then File.Delete(FileName)
                WriteStream = File.CreateText(FileName)
                For Each oCn In mCnList
                    With WriteStream
                        .WriteLine("{Begin Connection}")
                        .WriteLine("    Name:      " & oCn.Name)
                        .WriteLine("    FTP:       " & oCn.FTP)
                        .WriteLine("    Port:      " & oCn.Port)
                        .WriteLine("    Use:       " & oCn.UserName)
                        .WriteLine("    Password:  " & oCn.Password)
                        .WriteLine("    Transfer:  " & CStr(oCn.Transfer))
                        .WriteLine("    Type:      " & CStr(oCn.Connection))
                        .WriteLine("    Default:   " & CStr(oCn.DefaultConnection))
                        .WriteLine("{End Connection}")
                    End With
                Next
            Catch err As Exception
                MsgBox("Error while writing connections to file! " & err.Message, MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Exclamation + MsgBoxStyle.OKOnly, "TransData v1.00 (alpha)")
            Finally
                WriteStream.Close()
            End Try
        End If
        WriteToFile = mCnList.Count
    End Function

    Public Function DefaultConnection() As cConnection
        Dim cC As cConnection

        For Each cC In mCnList
            If cC.DefaultConnection Then
                Return cC
            End If
        Next
    End Function
End Class