Imports System.IO
Imports System.IO.File

Public Class cParam
    Public Enum Type As Byte
        parString
        parDecimal
        parReal
        parDate
        parBoolean
    End Enum

    Private mName As String
    Private mValue As Object
    Private mType As Type

    Public Property Name() As String
        Get
            Name = mName
        End Get
        Set(ByVal Value As String)
            If Value.Length > 0 Then
                mName = Value
            End If
        End Set
    End Property

    Public Property Value() As Object
        Get
            Try
                Select Case mType
                    Case Type.parBoolean
                        Value = CBool(mValue)
                    Case Type.parDecimal
                        Value = CLng(mValue)
                    Case Type.parReal
                        Value = CDbl(mValue)
                    Case Type.parDate
                        Value = CDate(mValue)
                    Case Else
                        Value = CStr(mValue)
                End Select
            Catch err As Exception
                Value = Nothing
            End Try
        End Get
        Set(ByVal NewValue As Object)
            mValue = NewValue
        End Set
    End Property

    Public Property ParameterType() As Type
        Get
            ParameterType = mType
        End Get
        Set(ByVal Value As Type)
            mType = Value
        End Set
    End Property
End Class

Public Class cParList
    Implements System.Collections.IEnumerable

    Private mCol As New Collection()

    Public Function Add(ByVal Name As String, ByVal Value As Object, ByVal ParType As cParam.Type) As cParam
        Dim oP As cParam

        If Name.Length > 0 Then
            oP = New cParam()
            With oP
                .Name = Name
                .Value = Value
                .ParameterType = ParType
            End With
            mCol.Add(oP, Name)
        End If
        Add = oP
    End Function

    Public Function Add(ByRef Parameter As cParam) As cParam
        Dim op As cParam

        op = New cParam()
        With op
            .Name = Parameter.Name
            .Value = Parameter.Value
            .ParameterType = Parameter.ParameterType
        End With
        mCol.Add(op, Parameter.Name)
        Add = op
    End Function

    Public Sub Remove(ByVal Name As String)
        mCol.Remove(Name)
    End Sub

    Public Sub Remove(ByVal Index As Integer)
        mCol.Remove(Index + 1)
    End Sub

    Public Function Clear() As Object
        Dim mIndex As Integer
        If mCol.Count() > 0 Then
            For mIndex = 1 To mCol.Count()
                mCol.Remove(mIndex)
            Next
        End If
    End Function

    Default Public ReadOnly Property Item(ByVal Index As Object) As cParam
        Get
            If IsNumeric(Index) Then
                Item = mCol.Item(Index + 1)
            Else
                Item = mCol.Item(Index)
            End If
        End Get
    End Property

    Public ReadOnly Property Count() As Integer
        Get
            Count = mCol.Count
        End Get
    End Property

    Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        GetEnumerator = mCol.GetEnumerator
    End Function
End Class

Public Class cParManager
    Private mParList As New cParList()

    Public Event FinishedLoading()

    Public Property Parameters() As cParList
        Get
            Parameters = mParList
        End Get
        Set(ByVal Value As cParList)

        End Set
    End Property

    Public Function ReadFromFile(ByVal FileName As String) As Integer
        If FileName.Length > 0 Then
            If Dir(FileName).Length > 0 Then
                Dim strLine As String = " "                     ' Must be a non-empty string for the loop below
                Dim ReadStream As StreamReader
                Dim oP As cParam
                Dim strTmp As String

                Try
                    ReadStream = File.OpenText(FileName)
                    strLine = ReadStream.ReadLine
                    Do While Not IsNothing(strLine)
                        strLine = strLine.Trim(" ")

                        If strLine.IndexOf("{Begin Parameter}") >= 0 Then
                            oP = New cParam()
                        ElseIf strLine.IndexOf("Name:") >= 0 Then
                            oP.Name = strLine.Substring(5)
                            oP.Name = oP.Name.Trim(" ")
                        ElseIf strLine.IndexOf("Value:") >= 0 Then
                            oP.Value = strLine.Substring(6).Trim(" ")
                        ElseIf strLine.IndexOf("Type:") >= 0 Then
                            strTmp = strLine.Substring(5)
                            strTmp = strTmp.Trim(" ")
                            oP.ParameterType = CByte(strTmp)
                        ElseIf strLine.IndexOf("{End Parameter}") >= 0 Then
                            If Not IsNothing(oP) Then
                                mParList.Add(oP)
                                oP = Nothing
                            End If
                        End If
                        strLine = ReadStream.ReadLine
                    Loop
                Catch eof As IO.EndOfStreamException
                    ' no action required
                Catch err As Exception
                    MsgBox("Error while reading the connections file! " & err.Message, MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Exclamation + MsgBoxStyle.OKOnly, "TransData v1.00 (alpha)")
                Finally
                    ReadStream.Close()
                End Try
                RaiseEvent FinishedLoading()
            Else
                FromScratch()
            End If
        Else
            FromScratch()
        End If
    End Function

    Public Function WriteToFile(ByVal FileName As String) As Integer
        Dim WriteStream As StreamWriter
        Dim oP As cParam

        If mParList.Count > 0 Then
            Try
                If Dir(FileName).Length > 0 Then File.Delete(FileName)
                WriteStream = File.CreateText(FileName)
                For Each oP In mParList
                    With WriteStream
                        .WriteLine("{Begin Parameter}")
                        .WriteLine("    Name:  " & oP.Name)
                        .WriteLine("    Value: " & CStr(oP.Value))
                        .WriteLine("    Type:  " & CStr(oP.ParameterType))
                        .WriteLine("{End Parameter}")
                    End With
                Next
            Catch err As Exception
                MsgBox("Error while writing connections to file! " & err.Message, MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Exclamation + MsgBoxStyle.OKOnly, "TransData v1.00 (alpha)")
            Finally
                WriteStream.Close()
            End Try
        End If
        WriteToFile = mParList.Count
    End Function

    Public Sub FromScratch()
        mParList.Clear()
        mParList.Add("ConnRetries", 3, cParam.Type.parDecimal)
        mParList.Add("TransferRetries", 3, cParam.Type.parDecimal)
        mParList.Add("UseCRC", True, cParam.Type.parBoolean)
        mParList.Add("UseHsk", True, cParam.Type.parBoolean)
        mParList.Add("DisplayInfo", True, cParam.Type.parBoolean)
        mParList.Add("UseLog", True, cParam.Type.parBoolean)
        mParList.Add("LogStore", eLog.el6Months, cParam.Type.parDecimal)
        mParList.Add("Protocol", Protocol.epFTP, cParam.Type.parDecimal)
        mParList.Add("Trigger", Trigger.etSchedule, cParam.Type.parDecimal)
        mParList.Add("DDP", "C:\", cParam.Type.parString)
        mParList.Add("ExternalApp", False, cParam.Type.parBoolean)
        mParList.Add("ExAppName", "NoName", cParam.Type.parString)
        mParList.Add("ExAppName1", "NoName", cParam.Type.parString)
    End Sub
End Class