Imports System.IO
Imports System.IO.File

Public Class cTask
    Public Enum tskDirection As Byte
        Upload
        Download
    End Enum

    Public Enum tskSchedule As Byte
        Daily
        SelectedWeekDays
        SpecificDateTime
    End Enum

    Private mFileName As String
    Private mSourcePath As String
    Private mDestPath As String
    Private mDirection As tskDirection
    Private mSchedule As tskSchedule
    Private mWeekDays As Short
    Private mDateTime As Date

    Public Property FileName() As String
        Get
            FileName = mFileName
        End Get
        Set(ByVal Value As String)
            If Value.Length > 0 Then FileName = Value
        End Set
    End Property

    Public Property SourcePath() As String
        Get
            SourcePath = mSourcePath
        End Get
        Set(ByVal Value As String)
            mSourcePath = Value
        End Set
    End Property

    Public Property DestPath() As String
        Get
            DestPath = mDestPath
        End Get
        Set(ByVal Value As String)
            mDestPath = Value
        End Set
    End Property

    Public Property Direction() As tskDirection
        Get
            Direction = mDirection
        End Get
        Set(ByVal Value As tskDirection)
            mDirection = Value
        End Set
    End Property

    Public Property Schedule() As tskSchedule
        Get
            Schedule = mSchedule
        End Get
        Set(ByVal Value As tskSchedule)
            mSchedule = Value
        End Set
    End Property

    Public Property WeekDays() As Short
        Get
            WeekDays = mWeekDays
        End Get
        Set(ByVal Value As Short)
            mWeekDays = Value
        End Set
    End Property

    Public Property DateTime() As Date
        Get
            DateTime = mDateTime
        End Get
        Set(ByVal Value As Date)
            mDateTime = Value
        End Set
    End Property
End Class

Public Class cTaskList
    Implements System.Collections.IEnumerable

    Private mCol As New Collection()

    Public Function Add(ByVal FileName As String, ByVal SourcePath As String, _
     ByVal DestPath As String, Optional ByVal Direction As cTask.tskDirection = cTask.tskDirection.Upload, Optional ByVal Schedule As cTask.tskSchedule = cTask.tskSchedule.Daily, _
     Optional ByVal WeekDays As Short = 0, Optional ByVal DateTime As Date = #1/1/1601#) As cTask
        Dim oNewTask As cTask

        If (FileName.Length > 0) Then
            oNewTask = New cTask()
            With oNewTask
                .FileName = FileName
                .SourcePath = SourcePath
                .DestPath = DestPath
                .Direction = Direction
                .Schedule = Schedule
                .WeekDays = WeekDays
                .DateTime = DateTime
            End With
            mCol.Add(oNewTask, FileName)
        End If
        Add = oNewTask
    End Function

    Public Function Add(ByRef Task As cTask) As cTask
        mCol.Add(Task, Task.FileName)
        Add = Task
    End Function

    Public Sub Remove(ByVal Name As String)
        mCol.Remove(Name)
    End Sub

    Public Sub Remove(ByVal Index As Integer)
        mCol.Remove(Index + 1)
    End Sub

    Public Function Clear() As Object
        Dim mIndex As Integer
        If mCol.Count() > 0 Then
            For mIndex = 1 To mCol.Count()
                mCol.Remove(mIndex)
            Next
        End If
    End Function

    Default Public ReadOnly Property Item(ByVal Index As Object) As cConnection
        Get
            If IsNumeric(Index) Then
                Item = mCol.Item(Index + 1)
            Else
                Item = mCol.Item(Index)
            End If
        End Get
    End Property

    Public ReadOnly Property Count() As Integer
        Get
            Count = mCol.Count
        End Get
    End Property

    Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        GetEnumerator = mCol.GetEnumerator
    End Function
End Class

Public Class cTaskManager
    Private mTaskList As New cTaskList()

    Public Event FinishedLoading()

    Public Property Connections() As cTaskList
        Get
            Connections = mTaskList
        End Get
        Set(ByVal Value As cTaskList)

        End Set
    End Property

    Public Function ReadFromFile(ByVal FileName As String) As Integer
        If FileName.Length > 0 Then
            If Dir(FileName).Length > 0 Then
                Dim strLine As String = " "                     ' Must be a non-empty string for the loop below
                Dim ReadStream As StreamReader
                Dim oTask As cTask
                Dim strTmp As String

                Try
                    ReadStream = File.OpenText(FileName)
                    strLine = ReadStream.ReadLine
                    Do While Not IsNothing(strLine)
                        strLine = strLine.Trim(" ")

                        If strLine.IndexOf("{Begin Task}") >= 0 Then
                            oTask = New cTask()
                        ElseIf strLine.IndexOf("FileName:") >= 0 Then
                            oTask.FileName = strLine.Substring(9).Trim(" ")
                        ElseIf strLine.IndexOf("Source:") >= 0 Then
                            oTask.SourcePath = strLine.Substring(7).Trim(" ")
                        ElseIf strLine.IndexOf("Destination:") >= 0 Then
                            oTask.DestPath = strLine.Substring(12).Trim(" ")
                        ElseIf strLine.IndexOf("Direction:") >= 0 Then
                            strTmp = strLine.Substring(10).Trim(" ")
                            oTask.Direction = CByte(strTmp)
                        ElseIf strLine.IndexOf("Schedule:") >= 0 Then
                            strTmp = strLine.Substring(9).Trim(" ")
                            oTask.Schedule = CByte(strTmp)
                        ElseIf strLine.IndexOf("WeekDays:") >= 0 Then
                            strTmp = strLine.Substring(9).Trim(" ")
                            oTask.WeekDays = CShort(strTmp)
                        ElseIf strLine.IndexOf("DateTime:") >= 0 Then
                            strTmp = strLine.Substring(9).Trim(" ")
                            oTask.DateTime = CDate(strTmp)
                        ElseIf strLine.IndexOf("{End Task}") >= 0 Then
                            If Not IsNothing(oTask) Then
                                mTaskList.Add(oTask)
                                oTask = Nothing
                            End If
                        End If
                        strLine = ReadStream.ReadLine
                    Loop
                Catch eof As IO.EndOfStreamException
                    ' no action required
                Catch err As Exception
                    MsgBox("Error while reading the connections file! " & err.Message, MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Exclamation + MsgBoxStyle.OKOnly, "TransData v1.00 (alpha)")
                Finally
                    ReadStream.Close()
                End Try
                RaiseEvent FinishedLoading()
            End If
        End If
    End Function

    Public Function WriteToFile(ByVal FileName As String) As Integer
        Dim WriteStream As StreamWriter
        Dim oTask As cTask

        If mTaskList.Count > 0 Then
            Try
                If Dir(FileName).Length > 0 Then File.Delete(FileName)
                WriteStream = File.CreateText(FileName)
                For Each oTask In mTaskList
                    With WriteStream
                        .WriteLine("{Begin Task}")
                        .WriteLine("    FileName:    " & oTask.FileName)
                        .WriteLine("    Source:      " & oTask.SourcePath)
                        .WriteLine("    Destination: " & oTask.DestPath)
                        .WriteLine("    Direction:   " & CStr(oTask.Direction))
                        .WriteLine("    Schedule:    " & CStr(oTask.Schedule))
                        .WriteLine("    WeekDays:    " & CStr(oTask.WeekDays))
                        .WriteLine("    DateTime:    " & CStr(oTask.DateTime))
                        .WriteLine("{End Connection}")
                    End With
                Next
            Catch err As Exception
                MsgBox("Error while writing tasks to file! " & err.Message, MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Exclamation + MsgBoxStyle.OKOnly, "TransData v1.00 (alpha)")
            Finally
                WriteStream.Close()
            End Try
        End If
        WriteToFile = mTaskList.Count
    End Function
End Class